import React, {useState, useEffect, useRef} from 'react'
import IconButton from '@mui/material/IconButton';
import ArrowBackIcon from '@mui/icons-material/ArrowBack';
import MoreVertIcon from '@mui/icons-material/MoreVert';
import Menu from '@mui/material/Menu';
import MenuItem from '@mui/material/MenuItem';

import { ChatAvatar, CChatModal } from "../components"

import { printEnding } from "../helpers"
import { removeUserChat } from "../actions"
import { connect } from 'react-redux'


const chatMngBody = {
   width: "100%",
   display: "flex",
   justifyContent: "flex-start",
   aligneItems: "center"
}
   const blockMobile = {
      flexShrink: 0,
      flexGrow: 0,
      flexBasis: "60px",
      display: "flex",
      justifyContent: "flex-start",
      aligneItems: "center",
      display: "none"
   }
   const blockLeft = {
      flexShrink: 1,
      flexGrow: 1,
      display: "flex",
      justifyContent: "flex-start",
      aligneItems: "center",
      cursor: "pointer"
   }
      const blockAv = {
         margin: "auto 0"
      }
      const blockInfo = {
         display: "flex",
         flexDirection: "column",
         justifyContent: "space-between",
         whiteSpace: "nowrap",
         textOverFlow: "ellipsis",
         marginLeft: "15px",
         userSelect: "none"
      }
         const blockName = {
            fontSize: "16px",
            fontWeight: "500",
         }
         const blockMemb = {
            fontSize: "14px",
            fontWeight: "300",
         }
   const blockRight = {
      flexShrink: 0,
      flexGrow: 0,
      flexBasis: "10%",
      display: "flex",
      justifyContent: "flex-end",
      aligneItems: "center"
   }



const chatMinInfo = ({ chat, OPEN }) => {
   if (chat) {
      return (
         <div style={blockLeft} onClick={OPEN}>
            <div style={blockAv}>
               <ChatAvatar chat={chat} />
            </div>
            <div style={blockInfo}>
               <div style={blockName}>
                  {chat.title}
               </div>
               <div style={blockMemb}>
                  {chat.members && `${chat.members.length} участник${printEnding(String(chat.members.length))}`} 
               </div>
            </div>
         </div>
      )
   } else {
      return (
         <></>
      )
   }
}




const ChatMenu = ({ chatId, onLeave }) => {
   const [anchorEl, setAnchorEl] = useState(null)
   const open = !!anchorEl

   const handleClick = (event) => {
     setAnchorEl(event.currentTarget)
   }

   const handleClose = () => {
     setAnchorEl(null)
   }

   return (
      <>
         <IconButton 
               style={{ color: '#fff' }} 
               onClick={handleClick}>
            <MoreVertIcon />
         </IconButton>
   
         <Menu
            anchorEl={anchorEl}
            open={open}
            onClose={handleClose}
         >

            <MenuItem onClick={handleClose}>
               Что то сделать
            </MenuItem>

            <MenuItem onClick={() => onLeave(chatId)}>
               Покинуть чат
            </MenuItem>
                                    
         </Menu>
      </>
   ) 
}
const CChatMenu = connect(null, {onLeave: removeUserChat})(ChatMenu)



const ChatMngHeader = ({ chats, chatId }) => {

   const chat = chats[chatId]
   // console.log(chat)
   return (
      <div style={chatMngBody}>

         <div style={blockMobile}>
            <IconButton style={{ color: '#fff' }} >
               <ArrowBackIcon />
            </IconButton>
         </div>


         <CChatModal key={chatId} create={false} chat={chat} render={chatMinInfo} /> 
         

         <div style={blockRight}>
            <CChatMenu chatId={chatId} />
         </div>
      </div>
   )
}
export const CChatMngHeader = connect(state => ( { chats: state.chats || []} ))(ChatMngHeader)


