import React, {useState, useEffect, useRef} from 'react';
import Box from '@mui/material/Box';
import Button from '@mui/material/Button';
import TextareaAutosize from '@mui/base/TextareaAutosize';
import SendIcon from '@mui/icons-material/Send';
import AttachFileIcon from '@mui/icons-material/AttachFile';
import CloseIcon from '@mui/icons-material/Close';

import {useDropzone} from 'react-dropzone';
import {render} from 'react-dom';
import {SortableContainer, SortableElement} from 'react-sortable-hoc';
import {arrayMoveImmutable} from 'array-move';

import { backURL }  from '../helpers'
import {connect}  from 'react-redux'
import {
   actionSendMsg
} from "../actions"

const containerWrapp = {
   display: 'flex', 
   flexDirection: 'column',
   padding: '5px',
}

const thumbsContainer = {
   flexGrow: '0',
   display: 'flex',
   flexDirection: 'row',
   flexWrap: 'wrap',
   width: '100%',
   overflow: 'auto',
 }
 
 const thumb = {
   flexGrow: '0',
   display: 'inline-flex',   
   borderRadius: 2,
   border: '1px solid #eaeaea',
   marginBottom: 8,
   marginRight: 8,
   width: 100,
   height: 118,
   padding: 4,
   boxSizing: 'border-box',
   position: 'relative',
   backgroundColor: '#fff'
 }
 
 const thumbInner = {
   paddingTop: 18,
   width: "100%",
   minWidth: 0,
   overflow: 'hidden'
 }

 const closeBtn = {
    position: 'absolute',
    right: 0,
 }

 const img = {
   margin: '0 auto',
   display: 'block',
   objectFit: "cover",
   maxWidth: 'auto',
   maxHeight: '100%'
 }

 const other = {
   margin: '0 auto',
   display: 'block',
   maxWidth: 'auto',
   maxHeight: '100%',
   userSelect: 'none',
 }
 const othTitle = {
   fontWeight: 500,
   fontSize: "14px"
 }
 const othContent = {
   marginTop: "2px",
   fontWeight: 300,
   fontSize: "12px"
}




 const SortableItem = SortableElement(({onDelete, file}) => (
   <div style={thumb}>         
      <div style={closeBtn}>
         <CloseIcon 
            fontSize="small" 
            sx={{ cursor: 'pointer' }}
            onClick={onDelete} 
         />
      </div>         
      <div style={thumbInner}>  
      { 
         (file.type.split('/')[0] === 'image') ? 
            <img
               src={file.url}
               style={img}
            />  :
            
            (file.type.split('/')[0] === 'video') ?
               <video 
                  autoPlay 
                  muted
                  loop
                  // не работает
                  // playbackrate={'10'}
                  src={file.url}
                  style={img}
               >
               </video> :

               <div
                  style={other}
               >
                  <div style={othTitle}>{file.type.split('/')[0]}</div>
                  <div style={othContent}>{file.name}</div>                  
               </div>
      }         

      </div>     
   </div>
 ));

 const SortableList = SortableContainer(({children}) => {

   return (
      <div style={thumbsContainer}>
         {children}
      </div>
   )
 });




const MsgDropZone = ({ setText, setFiles, setMsgId, files, text, onEnter }) => {

   const {
      acceptedFiles,
      getRootProps, 
      getInputProps
   } = useDropzone({
         noKeyboard: true,
         onDrop: async (acceptedFiles) => {
            // console.log(acceptedFiles)
            await setFiles([...files, 
               ...acceptedFiles.map(file => Object.assign(file, {
                  url: URL.createObjectURL(file)
               }))
            ])

         }
      });

   const onDelete = (i) => {
      setFiles( (files) => files.filter((el, index) => index !== i) )
   }

   const onSortEnd = ({oldIndex, newIndex}) => {
      setFiles( (files) => (
        arrayMoveImmutable(files, oldIndex, newIndex)
      ));
    }
   
   // useEffect(() => {
   // // Make sure to revoke the data uris to avoid memory leaks
   // files.forEach(file => URL.revokeObjectURL(file.url));
   // }, [files]);
   const textArea = useRef(null)

   useEffect(() => {
      textArea.current.focus()
   })

  return (
   <>
      <section style={containerWrapp}>

         <SortableList 
            onSortEnd={onSortEnd}
            axis={'xy'}
            pressDelay={100}
         >
            {files.map((file, i) => (
               <SortableItem 
                  key={file.url} 
                  index={i} 
                  file={file}
                  onDelete={() => onDelete(i)}
                  axis={'xy'}
               />
            ))}
         </SortableList>
      

         <div {...getRootProps({className: 'dropzone'})} style={{ display: 'flex', alignItems: 'center'}}>
               <input {...getInputProps()} type="file" name="media" id='media'/>

               <AttachFileIcon fontSize="large" sx={{ cursor: 'pointer' }} />
               
               <TextareaAutosize
                  ref={textArea}  
                  value={text}                   
                  minRows={4}
                  maxRows={10}
                  placeholder="Написать сообщение..."
                  style={{ width: '100%' }}
                  onClick={(e) => e.stopPropagation()}
                  onChange={(e) => setText(e.target.value)}
                  onKeyPress={(e) => {
                     if (e.key === 'Enter' && !e.shiftKey) { 
                        e.preventDefault()   
                        ;(text.match(/^\s*$/) && files.length === 0) ||
                              onEnter()
                        setText('')
                        setFiles([])
                        setMsgId()
                     }
                  }}
               />
         </div>

      </section>
   </>
  )
}


const SendingField = ({ chatId, onSend, msg }) => {

   const [text, setText] = useState(msg?.text || '')
   const [files, setFiles] = useState(msg?.media.map(mediaFile =>( {...mediaFile, url: backURL +  mediaFile.url} )) || [])
   const [msgId, setMsgId] = useState(msg?._id || null)


   useEffect(() => {
      setText(msg?.text || '')
      setFiles(msg?.media.map(mediaFile =>( {...mediaFile, url: backURL +  mediaFile.url} )) || [])
      setMsgId(msg?._id || null)
   },[msg])

   return (
      <Box sx={{ display: 'flex', alignItems: 'stratch', flexDirection: 'column',
            height: '100%', width: '100%'}} >
      
         <Box sx={{ flexGrow: 1, flexShrink: 1, overflow: 'auto', backgroundColor: '#fff' }}>

            { msgId &&
               <Box sx={{ flexGrow: 1, flexShrink: 1}}>
                  <Button 
                     sx={{ width: "100%" }}
                     variant="contained" 
                     onClick={() => {
                        setText('')
                        setFiles([])
                        setMsgId()
                     }}
                  >
                     Отменить редактирование
                  </Button>
               </Box>
            }
            
            <MsgDropZone 
               setText={setText} 
               setFiles={setFiles} 
               setMsgId={setMsgId}
               files={files} 
               text={text} 
               onEnter={() => onSend(chatId, text.trim(), "media", files, msgId)} />
         
         </Box>         
         <Box sx={{ flexGrow: 1, flexShrink: 1}}>
            <Button 
               sx={{ width: "100%" }}
               variant="contained" 
               endIcon={<SendIcon />} 
               onClick={() => {
                  (text.match(/^\s*$/) && files.length === 0) ||
                        onSend(chatId, text.trim(), "media", files, msgId) 
                  setText('')
                  setFiles([])
                  setMsgId()
               }}
            >
               Отправить
            </Button>
         </Box>
         
      </Box>
   )
}
export const CSendingField= connect( null,
                           {onSend: actionSendMsg})(SendingField)