import { createStore, combineReducers, applyMiddleware } from 'redux'
import thunk from 'redux-thunk'
import { promiseReducer } from './promiseReducer'
import { authReducer } from './authReducer'
import { chatsReducer } from './chatsReducer'
import { localStoredReducer } from './localStoredReducer'

import { actionAboutMe } from './findUserActions'

import {   
   actionOnMsg,
   actionOnChat,
   actionOnChatLeft,
   actionFullLogout,
} from '../actions'


let initialState = localStorage.getItem('state')

export const store =  createStore ( combineReducers({ 
                                       auth: authReducer,
                                       chats: chatsReducer, 
                                       promise: promiseReducer, 
                                                }, 
                                                ),
                                    initialState ? {chats: JSON.parse(initialState)} : {},
                                    applyMiddleware(thunk),
                        )

store.dispatch(actionAboutMe())

// store.subscribe(() => console.log(store.getState()))
window.onbeforeunload = () => window.localStorage.setItem('state', JSON.stringify(store.getState().chats) )

///////////////////////////////////////////////////////////////////

export const socket = window.io("ws://chat.fs.a-level.com.ua")

socket.on('jwt_ok',   (data)  => console.log(data))
socket.on('jwt_fail', (error) => {
   console.log(error)
   store.dispatch(actionFullLogout())
})

socket.on('msg', (msg) => { 
   console.log('пришло смс')
   store.dispatch(actionOnMsg(msg))
})

socket.on('chat', (chat) => { 
   console.log('нас добавили в чат')
   store.dispatch(actionOnChat(chat))

   // нужно переподключать сокет, так как после добавления 
   // в чат он не работает, если юзер уже был в этом чате мембером
   let state = store.getState()
   socket.disconnect(true)
   socket.connect()
   socket.emit('jwt', state.auth.token)
})

socket.on('chat_left', (chat) => { 
   console.log('нас выкинули из чата')
   store.dispatch(actionOnChatLeft(chat))
})





